local rt = RoosterTools
local db = {}

local defaultDb = {
    partyGuid = nil,
    config = {
        raid = {
            minGuildMembers = 10
        },
        party = {
            minGuildMembers = 4
        },
        sync = {
            refuseRankBelow = 5 -- Below actually indicates greater given 0 is the highest guild rank. 
        }
    },
    characters = {},
    charactersIndex = {
        byName = {}
    },
    groups = {},
    mains = {},
    sync = {},
    aliases = {},
    debug = false,
}

RoosterToolsDBConn = {}

function RoosterToolsDBConn:SelectCharacterByName(name) 
    local guid = db.charactersIndex.byName[name]
    local player = db.characters[guid]

    if (player) then
        return guid, player
    else
        return nil
    end
end

function RoosterToolsDBConn:SelectGuildRaidGroups() 
    local t = {}
    
    for k, v in pairs(db.groups) do
        if (v.guildGroupTime and v.type == "raid") then
            for k, v in pairs(v.instances) do
                -- dump some of the weird raid events that have 'pvp' as the instance type. 
                if (v.type == "raid") then
                    t[k] = v
                end
            end
        end
    end

    return t
end

function RoosterToolsDBConn:SelectGuildGroups() 
    local t = {}
    
    for k, v in pairs(db.groups) do
        if (v.guildGroupTime) then
            t[k] = v
        end
    end

    return t
end

function RoosterToolsDBConn:UpsertCharacter(guid, character) 
    db.characters[guid] = character

    if (character.unitName) then
        db.charactersIndex.byName[character.unitName] = guid
    end
end

function RoosterToolsDBConn:OnInitialize() 
    db = RoosterToolsDB

     -- Init the db if it hasn't been used.
     if (not db) then
        RoosterToolsDB = defaultDb
        db = RoosterToolsDB
    end

    if (not db.aliases) then
        -- DB Upgrade
        db.aliases = {}
    end

    if (not db.debug) then
        db.debug = false
    end

    if (not db.config.sync or not db.config.sync.refuseRankBelow) then
        db.config.sync = defaultDb.config.sync
    end

    -- Create a unique id for this addon. 
    if(not db.id) then
        db.id = time() .. rt.player.guid
    end

    if (not db.charactersIndex) then
        db.charactersIndex = {}
        db.charactersIndex.byName = {}

        -- Build initial index.
        for k, v in pairs(db.characters) do 
            if (v.unitName) then
                db.charactersIndex.byName[v.unitName] = k
            end
        end
    end

    db.info = nil
    db.groupsSerialized = nil
end
